<?php

declare(strict_types=1);

namespace Drupal\l10n_community\Controller;

use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Config\ConfigManagerInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Render\Renderer;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\group\Entity\Group;
use Drupal\group\Entity\GroupInterface;
use Drupal\l10n_community\L10nTranslator;
use Drupal\l10n_server\Entity\L10nServerProject;
use Drupal\l10n_server\Entity\L10nServerProjectInterface;
use Drupal\l10n_server\Entity\L10nServerReleaseInterface;
use Drupal\l10n_server\Entity\L10nServerStringInterface;
use Drupal\l10n_server\Entity\L10nServerTranslationInterface;
use Drupal\l10n_server\L10nPo;
use Drupal\language\Entity\ConfigurableLanguage;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;

/**
 * Returns responses for Localization community UI routes.
 */
final class L10nCommunityLanguagesController extends ControllerBase {

  /**
   * The controller constructor.
   *
   * @param \Drupal\Core\Database\Connection $connection
   *   The database connection.
   * @param \Drupal\Core\Config\ConfigManagerInterface $configManager
   *   The config manager.
   * @param \Drupal\Core\Render\Renderer $renderer
   *   The renderer service.
   * @param \Drupal\l10n_community\L10nTranslator $translator
   *   The translator service.
   * @param \Drupal\l10n_server\L10nPo $l10n_po
   *   L10n helper.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request
   *   The request stack.
   */
  public function __construct(
    private readonly Connection $connection,
    private readonly ConfigManagerInterface $configManager,
    private readonly Renderer $renderer,
    private readonly L10nTranslator $translator,
    private readonly L10nPo $l10n_po,
    private readonly RequestStack $request,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
  ): self {
    return new L10nCommunityLanguagesController(
      $container->get('database'),
      $container->get('config.manager'),
      $container->get('renderer'),
      $container->get('l10n_community.translator'),
      $container->get('l10n_server.po'),
      $container->get('request_stack'),
    );
  }

  /**
   * Order listing table by column for language overview columns.
   */
  public static function sortByColumnLanguage($a, $b): int {
    $sortkey = ($_GET['order'] == t('Language') ? 0 : ($_GET['order'] == t('Contributors') ? 2 : 1));
    if (@$a[$sortkey]['sortdata'] == @$b[$sortkey]['sortdata']) {
      return 0;
    }
    return ((@$a[$sortkey]['sortdata'] < @$b[$sortkey]['sortdata']) ? -1 : 1) * ($_GET['sort'] == 'asc' ? 1 : -1);
  }

  /**
   * Builds the response.
   */
  public function explore(): array {
    /** @var \Drupal\l10n_community\L10nStatistics $statistics */
    $statistics = \Drupal::service('l10n_community.statistics');

    // Checking whether we have languages to translate to.
    if (!$languages = \Drupal::languageManager()->getLanguages()) {
      $build['content'] = [
        '#type' => 'item',
        '#markup' => $this->t('No languages to list.'),
      ];
      return $build;
    }

    // Checking whether we have strings to translate.
    if (!$num_source = $statistics->getStringCount()) {
      $build['content'] = [
        '#type' => 'item',
        '#markup' => $this->t('No strings to translate.'),
      ];
      return $build;
    }

    // Generate listing of all languages with summaries. The list of languages
    // is relatively "short", compared to projects, so we don't need a pager
    // here.
    $table_rows = [];
    $string_counts = $statistics->getLanguagesStringCount();
    foreach ($languages as $langcode => $language) {
      // Need to load this again to get access to the third party settings :/.
      $language = ConfigurableLanguage::load($langcode);

      if (!$language->getThirdPartySetting('l10n_pconfig', 'formula')) {
        $table_rows[] = [
          [
            'data' => t('@language', [
              '@language' => $language->getName(),
            ]),
            'sortdata' => t('@language', [
              '@language' => $language->getName(),
            ]),
            'class' => ['rowhead'],
          ],
          [
            'data' => t('Uninitialized plural formula. Please set up the plural formula in <a href="@language-config">the langauge configuration</a> or alternatively <a href="@import-url">import a valid interface translation</a> for Drupal in this language.', [
              '@import-url' => Url::fromUri('internal:/admin/structure/translate/import')->toString(),
              '@language-config' => Url::fromUri('internal:/admin/config/regional/language')->toString(),
            ]),
            'class' => ['error'],
          ],
          ['data' => ''],
        ];
      }
      else {
        $stats = $statistics->getLanguageStatisticsByLanguage($langcode);
        $progress = [
          'data' => [
            '#theme' => 'l10n_community_progress_columns',
            '#sum' => $stats['strings'],
            '#translated' => $stats['translations'],
            '#has_suggestion' => $stats['suggestions'],
          ],
        ];
        $table_rows[] = [
          [
            'data' => new FormattableMarkup('<a href=":link">@name</a>', [
              ':link' => Url::fromUri('internal:/translate/languages/' . $langcode)->toString(),
              '@name' => $language->getName(),
            ]),
            'sortdata' => t('@language', [
              '@language' => $language->getName(),
            ]),
            'class' => ['rowhead'],
          ],
          [
            'data' => $progress,
            'sortdata' => ($num_source == 0 ? 0 : round(@$string_counts[$langcode]['translations'] / $num_source * 100, 2)),
          ],
          [
            'data' => $stats['users'],
            'sortdata' => $stats['users'],
          ],
        ];
      }
    }

    if (!empty($_GET['sort']) && !empty($_GET['order'])) {
      usort($table_rows, static::class . '::sortByColumnLanguage');
    }

    $header = [
      [
        'data' => t('Language'),
        'class' => ['rowhead'],
        'field' => 'language',
      ],
      [
        'data' => t('Overall progress'),
        'field' => 'progress',
      ],
      [
        'data' => t('Contributors'),
        'field' => 'contributors',
      ],
    ];
    $build['content'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $table_rows,
      '#attributes' => [
        'class' => [
          'l10n-community-overview l10n-community-highlighted',
        ],
      ],
      '#cache' => [
        'max-age' => Cache::PERMANENT,
      ],
    ];
    $build['#attached']['library'][] = 'l10n_community/tables';
    return $build;
  }

  /**
   * Builds the response.
   *
   * @param \Drupal\group\Entity\Group $group
   *   The group entity.
   *
   * @return array
   *   An associative render array.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  public function translate(Group $group): array {
    /** @var \Drupal\l10n_community\L10nTranslator $translator */
    $translator = \Drupal::service('l10n_community.translator');
    $langcode = $group->get('field_translation_language')->first()->getValue()['target_id'];
    $language = $this->languageManager()->getLanguage($langcode);

    $filters = self::buildFilterValues($_GET);
    $drupal_settings = [
      'l10nServerURLs' => $this->addUrlModifiers($langcode, $filters),
    ];
    $strings = $translator->getStrings($langcode, $filters, $filters['limit']);

    // Add RTL style if the current language's direction is RTL.
    if ($language->getDirection() == LanguageInterface::DIRECTION_RTL) {
      $build['#attached']['library'][] = 'l10n_community/editor-rtl';
    }

    // Set the most appropriate title.
    if ($filters['project']) {
      $project = '';

      if ($project) {
        $build['#title'] = $this->t('Translate %project to @language', [
          '%project' => $project->label(),
          '@language' => $language->getName(),
        ]);
      }
    }
    else {
      $build['#title'] = $this->t('Translate to @language', [
        '@language' => $language->getName(),
      ]);
    }

    // Add the filter form.
    $build['filter'] = $this->formBuilder()->getForm('Drupal\l10n_community\Form\FilterForm');

    // Output the actual strings.
    if (!count($strings)) {
      $this->messenger()->addError($this->t('No strings found with this filter. Try adjusting the filter options.'));
    }
    else {
      $build['translate_form'] = $this->formBuilder()->getForm('Drupal\l10n_community\Form\TranslateForm', $language, $filters, $strings);
    }

    $build['#attached']['drupalSettings'] = $drupal_settings;

    return $build;
  }

  /**
   * Title callback.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   *   Page title.
   */
  public function translateTitle(): TranslatableMarkup {
    return $this->t('Translate');
  }

  /**
   * Builds the response.
   *
   * @param \Drupal\group\Entity\Group $group
   *   The group entity.
   *
   * @return array
   *   An associative render array.
   */
  public function import(Group $group): array {
    return $this->formBuilder()
      ->getForm(
        'Drupal\l10n_community\Form\ImportForm',
        $group->get('field_translation_language')->first()->getValue()['target_id']
      );
  }

  /**
   * Title callback.
   *
   * @param \Drupal\group\Entity\GroupInterface $group
   *   The group entity.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   *   The page title.
   *
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  public function importTitle(GroupInterface $group): TranslatableMarkup {
    $language = $this->languageManager()->getLanguage($group->get('field_translation_language')->first()->getValue()['target_id']);

    return $this->t(
      'Import to @language',
      [
        '@language' => $language->getName(),
      ]
    );
  }

  /**
   * Title callback.
   *
   * @param \Drupal\group\Entity\GroupInterface $group
   *   The group entity.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   *   The page title.
   *
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  public function exportTitle(GroupInterface $group): TranslatableMarkup {
    $language = $this->languageManager()->getLanguage($group->get('field_translation_language')->first()->getValue()['target_id']);

    return $this->t(
      'Export @language translations',
      [
        '@language' => $language->getName(),
      ]
    );
  }

  /**
   * Builds the response.
   *
   * @param \Drupal\group\Entity\Group $group
   *   The group object.
   * @param \Drupal\l10n_server\Entity\L10nServerProjectInterface $project
   *   The project object.
   * @param \Drupal\l10n_server\Entity\L10nServerReleaseInterface $release
   *   The release object.
   *
   * @return array
   *   A render array.
   *
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  public function reset(Group $group, L10nServerProjectInterface $project, L10nServerReleaseInterface $release): array {
    $language = \Drupal::languageManager()->getLanguage($group->get('field_translation_language')->first()->getValue()['target_id']);

    $build['content'] = [
      '#type' => 'item',
      '#markup' => __METHOD__ . '::' . $group->label() . '::' . $project->label() . '::' . $release->label() . '::' . $language->getId(),
    ];

    return $build;
  }

  /**
   * Check and sanitize arguments and build filter array.
   *
   * @param array $params
   *   Associative array with unsanitized values.
   *
   * @return array
   *   Return filter values.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public static function buildFilterValues(array $params): array {
    $project = $release = NULL;

    // Convert array representation of flags to one integer.
    if (isset($params['status']) && is_array($params['status'])) {
      if (isset($params['status']['suggestion'])) {
        $params['status'] = ((int) $params['status']['translation']) | ((int) $params['status']['suggestion']);
      }
      else {
        $params['status'] = (int) $params['status']['translation'];
      }
    }

    $filter = [
      'project' => NULL,
      'status' => isset($params['status']) ? (int) $params['status'] : 0,
      'release' => 'all',
      'search' => !empty($params['search']) ? (string) $params['search'] : '',
      'author' => NULL,
      // Dropdown, validated by form API.
      'context' => isset($params['context']) ? (string) $params['context'] : 'all',
      'limit' => (
        isset($params['limit'])
        && in_array($params['limit'], [5, 10, 20, 30, 50])
      )
        ? (int) $params['limit']
        : 10,
      'sid' => (!empty($params['sid']) && is_numeric($params['sid'])) ? $params['sid'] : 0,
    ];

    if (isset($params['author'])) {
      $user_storage = \Drupal::entityTypeManager()
        ->getStorage('user');
      if (is_numeric($params['author'])) {
        $user = $user_storage->load($params['author']);
      }
      else {
        $users = $user_storage->loadByProperties(['name' => $params['author']]);
        $user = reset($users);
      }
      if (isset($user)) {
        $filter['author'] = $user->id();
      }
    }

    // The project can be a dropdown or text field depending on number of
    // projects. So we need to sanitize its value.
    if (isset($params['project'])) {
      // Try to load project by uri or id, but give URI priority. URI is used
      // to shorten the URL and have simple redirects. ID is used if the
      // filter form was submitted.
      $project_storage = \Drupal::entityTypeManager()
        ->getStorage('l10n_server_project');

      if (is_numeric($params['project'])) {
        $project = $project_storage->load($params['project']);
      }
      else {
        $projects = $project_storage->loadByProperties(['uri' => $params['project']]);
        $project = reset($projects);
      }

      /** @var \Drupal\l10n_server\Entity\L10nServerProject $project */
      if ($project instanceof L10nServerProject) {
        $filter['project'] = $project->id();

        $release_storage = \Drupal::entityTypeManager()
          ->getStorage('l10n_server_release');
        if (
          isset($params['release'])
          && ($releases = $release_storage->loadByProperties(
            [
              'pid' => $project->id(),
              'rid' => $params['release'],
            ],
          ))
        ) {
          // Allow to select this release, if belongs to current project only.
          $filter['release'] = $params['release'];
        }
      }
    }
    return $filter;
  }

  /**
   * Generate and add JS for URL replacements.
   *
   * These ensure we keep filter values.
   *
   * @param string $langcode
   *   Language code.
   * @param array $filters
   *   Associative array with filters passed.
   *
   * @return array
   *   Return URL replacements list.
   */
  public function addUrlModifiers(string $langcode, array $filters): array {
    $urls = [
      'translate/languages/' . $langcode . '/translate',
      'translate/languages/' . $langcode . '/export',
    ];
    $replacements = [];

    foreach ($urls as $url) {
      $replacements[Url::fromUri('internal:/' . $url)->toString()] = Url::fromUri('internal:/' . $url, ['query' => $filters])->toString();
    }

    return $replacements;
  }

  /**
   * Provides full history information about translation.
   *
   * This callback is invoked from AJAX.
   *
   * @param \Drupal\l10n_server\Entity\L10nServerTranslationInterface $translation
   *   Translation entity.
   *
   * @return array|Response
   *   Response to call.
   *
   * @throws \Exception
   */
  public function translationDetails(L10nServerTranslationInterface $translation): array|Response {
    $history_item_list = [];
    $string_entity = $translation->get('sid')->referencedEntities();
    $string = array_pop($string_entity);
    /** @var \Drupal\l10n_server\Entity\L10nServerTranslationHistory $translation_history_item */
    foreach ($translation->getHistory() as $translation_history_item) {
      $history_item_list[] = $this->l10nPo->translateByline(
        $translation_history_item->getAuthorName(),
        $translation_history_item->getAuthorId(),
        $translation_history_item->getDate(),
        $translation_history_item->getMedium(),
        $translation_history_item->getType()
      );
    }

    $build = [
      '#theme' => 'item_list',
      '#items' => $history_item_list,
    ];

    // Handle ajax requests that display usage in the translation form.
    if ($this->request->getCurrentRequest()->isXmlHttpRequest()) {
      return (new Response())
        ->setContent($this->renderer->renderPlain($build)->jsonSerialize());
    }

    return [
      'source_string' => [
        [
          '#markup' => '<h3>' . $this->t('Translation') . '</h3>',
        ],
        [
          '#theme' => 'item_list',
          '#items' => [$translation->get('translation')->getString()],
        ],
      ],
      'used_at' => [
        [
          '#markup' => '<h3>' . $this->t('History') . '</h3>',
        ],
        $build,
      ],
    ];
  }

  /**
   * Generate a list of projects and releases where a string appears.
   *
   * We could provide much more information (down to line numbers of files), but
   * usability should also be kept in mind. It is possible to investigate hidden
   * information sources though, like tooltips on the release titles presented.
   *
   * This callback is invoked from AJAX.
   *
   * @param \Drupal\l10n_server\Entity\L10nServerStringInterface $string
   *   String entity.
   *
   * @return array|Response
   *   Response to call.
   */
  public function sourceDetails(L10nServerStringInterface $string): array|Response {
    $version_list = [];
    $project_list = [];
    $previous_project = '';

    foreach ($this->translator->getSourceDetails($string->id()) as $instance) {
      $release_info = $instance->version . ' <span title="' . $this->formatPlural($instance->occurrence_count, 'Appears once in this release.', 'Appears @count times in this release.') . '">(' . $instance->occurrence_count . ')</span>';

      if ($instance->project_title != $previous_project) {
        if (!empty($version_list)) {
          $project_list[] = ['#markup' => implode(', ', $version_list)];
        }
        $version_list = ['<em>' . $instance->project_title . ':</em> ' . $release_info];
      }
      else {
        $version_list[] = $release_info;
      }

      $previous_project = $instance->project_title;
    }

    $project_list[] = ['#markup' => implode(', ', $version_list)];
    $build = [
      '#theme' => 'item_list',
      '#items' => $project_list,
    ];

    // Handle ajax requests that display usage in the translation form.
    if ($this->request->getCurrentRequest()->isXmlHttpRequest()) {
      return (new Response())
        ->setContent($this->renderer->renderPlain($build)->jsonSerialize());
    }

    return [
      'source_string' => [
        [
          '#markup' => '<h3>' . $this->t('Source string') . '</h3>',
        ],
        [
          '#theme' => 'item_list',
          '#items' => [$string->getValue()],
        ],
      ],
      'used_at' => [
        [
          '#markup' => '<h3>' . $this->t('Used at the following places') . '</h3>',
        ],
        $build,
      ],
    ];
  }

}
