<?php

namespace Drupal\Tests\l10n_migrate\Kernel;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Tests\migrate_drupal\Kernel\MigrateDrupalTestBase;

/**
 * Base class for Localization migration tests.
 */
abstract class MigrateL10nTestBase extends MigrateDrupalTestBase {

  /**
   * The entity storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   *   The entity storage
   */
  protected EntityStorageInterface $entityStorage;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->entityStorage = $this->container->get('entity_type.manager')
      ->getStorage(static::ENTITY_TYPE);
    $this->installConfig(['l10n_migrate', 'node', 'system']);
    $this->loadFixture($this->getFixtureFilePath(static::ENTITY_TYPE));
    $this->installEntitySchema(static::ENTITY_TYPE);
  }

  /**
   * Gets the path to the fixture file.
   *
   * @var string $entity_type
   *   The entity type string.
   */
  protected function getFixtureFilePath(string $entity_type): string {
    return __DIR__ . '/../../fixtures/localize7_' . $entity_type . '.php';
  }

  /**
   * Assert all rows have been migrated.
   */
  protected function assertRowsComplete(): void {
    $db = \Drupal::database();
    $query = $db->select(static::ENTITY_TYPE, 'l10n');
    $this->assertSame(count(static::expectedDataTable()), (int) $query->countQuery()->execute()->fetchField());
  }

  /**
   * Assert all datasets are complete.
   */
  protected function assertDatasetsComplete(): void {
    $db = \Drupal::database();
    $field_aliases = static::DATABASE_FIELDS_D7;
    $query = $db->select(static::ENTITY_TYPE, 'l10n');
    foreach (static::DATABASE_FIELDS_D10 as $field_name) {
      $field_alias = array_shift($field_aliases);
      $query->addField('l10n', $field_name, $field_alias);
    }
    $result = $query->execute()
      ->fetchAll(\PDO::FETCH_ASSOC);
    $this->assertEquals(static::expectedDataTable(), $result);
  }

  /**
   * Asserts all entities are valid.
   */
  protected function assertEntitiesValid(): void {
    // Confirm resulting entities are valid.
    $entities = $this->entityStorage->loadMultiple();
    foreach ($entities as $entity) {
      $this->assertInstanceOf(static::ENTITY_CLASS, $entity);
    }
  }

  /**
   * Tests migration for completeness.
   */
  public function testRowsComplete(): void {
    $this->assertRowsComplete();
  }

  /**
   * Tests migration for completeness.
   */
  public function testDatasetsComplete(): void {
    $this->assertDatasetsComplete();
  }

  /**
   * Tests migrated data for validity.
   */
  public function testEntitiesValid(): void {
    $this->assertEntitiesValid();
  }

  /**
   * Tests migrated data for validity.
   */
  public function testEntityAttributesValid(): void {
    $this->assertEntityAttributesValid();
  }

  /**
   * Asserts all entity attributes are valid.
   */
  abstract protected function assertEntityAttributesValid(): void;

  /**
   * Provides the expected data table.
   *
   * @return array
   *   The expected table rows.
   */
  abstract public static function expectedDataTable(): array;

}
