<?php

declare(strict_types=1);

namespace Drupal\l10n_server\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;

/**
 * Defines the history entity class.
 *
 * @ContentEntityType(
 *   id = "l10n_server_translation_history",
 *   label = @Translation("History"),
 *   label_collection = @Translation("Histories"),
 *   label_singular = @Translation("history"),
 *   label_plural = @Translation("histories"),
 *   label_count = @PluralTranslation(
 *     singular = "@count histories",
 *     plural = "@count histories",
 *   ),
 *   handlers = {
 *     "list_builder" = "Drupal\l10n_server\Entity\ListBuilder\L10nServerTranslationHistoryListBuilder",
 *     "storage_schema" = "Drupal\l10n_server\Entity\Storage\L10nServerTranslationHistoryStorageSchema",
 *     "views_data" = "Drupal\views\EntityViewsData",
 *     "form" = {
 *       "add" = "Drupal\l10n_server\Form\L10nServerTranslationHistoryForm",
 *       "edit" = "Drupal\l10n_server\Form\L10nServerTranslationHistoryForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm",
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\l10n_server\Routing\L10nServerTranslationHistoryHtmlRouteProvider",
 *     }
 *   },
 *   base_table = "l10n_server_translation_history",
 *   admin_permission = "administer l10n server translation history",
 *   entity_keys = {
 *     "id" = "thid",
 *     "label" = "uuid",
 *     "uuid" = "uuid",
 *   },
 *   links = {
 *     "collection" = "/admin/localization/translation-history",
 *     "add-form" = "/translation-history/add",
 *     "canonical" = "/translation-history/{l10n_server_translation_history}",
 *     "edit-form" = "/translation-history/{l10n_server_translation_history}",
 *     "delete-form" = "/translation-history/{l10n_server_translation_history}/delete",
 *   },
 * )
 */
class L10nServerTranslationHistory extends ContentEntityBase implements L10nServerTranslationHistoryInterface {

  /**
   * Action flag for string addition.
   */
  const ACTION_ADD = 1;

  /**
   * Action flag for approval.
   */
  const ACTION_APPROVE = 2;

  /**
   * Action flag for denial.
   */
  const ACTION_DECLINE = 3;

  /**
   * Action flag for demotes.
   */
  const ACTION_DEMOTE = 4;

  /**
   * Action flag for re-additions.
   */
  const ACTION_READD = 5;

  /**
   * Action medium flag for unknown sources.
   */
  const MEDIUM_UNKNOWN = 0;

  /**
   * Action medium flag for web based actions.
   */
  const MEDIUM_WEB = 1;

  /**
   * Action medium flag for web based import.
   */
  const MEDIUM_IMPORT = 2;

  /**
   * Action medium flag for remote action.
   */
  const MEDIUM_REMOTE = 3;

  /**
   * Used to mark counting duplicates.
   */
  const COUNT_DUPLICATE = 'duplicate';

  /**
   * Used to mark counting unchanged strings.
   */
  const COUNT_UNCHANGED = 'unchanged';

  /**
   * Used to mark counting ignored strings.
   */
  const COUNT_IGNORED = 'ignored';

  /**
   * Used to mark counting suggested strings.
   */
  const COUNT_SUGGESTED = 'suggested';

  /**
   * Used to mark counting added strings.
   */
  const COUNT_ADDED = 'added';

  /**
   * Used to mark counting updated strings.
   */
  const COUNT_UPDATED = 'updated';

  /**
   * Used to mark counting declined strings.
   */
  const COUNT_DECLINED = 'declined';

  /**
   * Used to mark counting declined suggestions.
   */
  const COUNT_SUGGESTION_DECLINED = 'suggestion_declined';

  /**
   * Used to mark counting approved suggestions.
   */
  const COUNT_APPROVED = 'approved';

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type): array {
    $fields = parent::baseFieldDefinitions($entity_type);
    $fields['tid'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('ranslation'))
      ->setRequired(TRUE)
      ->setDescription(t('Reference to the {l10n_server_translation}.tid.'))
      ->setSetting('target_type', 'l10n_server_translation')
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('view', TRUE);
    $fields['uid_action'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Author'))
      ->setDescription('Reference to the {users}.uid who performed the action.')
      ->setSetting('target_type', 'user')
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => 60,
          'placeholder' => '',
        ],
        'weight' => 0,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'author',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('view', TRUE);
    $fields['time_action'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Authored on'))
      ->setDescription(t('Unix timestamp of time when the action happened.'))
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'timestamp',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'datetime_timestamp',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('view', TRUE);
    $fields['type_action'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Type action'))
      ->setDescription(t('Numeric identifier of the action that happened.'))
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'number_integer',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => 0,
      ]);
    $fields['medium_action'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Medium action'))
      ->setDescription(t('Numeric identifier of the medium the action happened through.'))
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'number_integer',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => 0,
      ]);
    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function getActionUid(): ?int {
    return (int) $this->get('uid_action')->first()->getValue()['target_id'];
  }

  /**
   * Get the author of the translation object.
   *
   * @return \Drupal\user\UserInterface
   *   Return the author of the translation object.
   */
  public function getAuthor(): EntityInterface {
    return $this->get('uid_action')->entity;
  }

  /**
   * Get the name of the translator.
   *
   * @return string
   *   Return the name of the translator.
   */
  public function getAuthorName(): string {
    return $this->getAuthor()->label();
  }

  /**
   * Get the id of the translator.
   *
   * @return int
   *   Return the id of the translator.
   */
  public function getAuthorId(): int {
    return (int) $this->getAuthor()->id();
  }

  /**
   * Get the date of the translation.
   *
   * @return int|null
   *   Return the date of the translation.
   *
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  public function getDate(): ?int {
    return (int) $this->get('time_action')->first()?->getString();
  }

  /**
   * Get the medium of the translation.
   *
   * @return int|null
   *   Return the medium of the translation.
   *
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  public function getMedium(): ?int {
    return (int) $this->get('medium_action')->first()?->getString();
  }

  /**
   * Get the action type of the translation.
   *
   * @return int
   *   Return the action type of the translation.
   *
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  public function getType(): int {
    return (int) $this->get('type_action')->first()?->getString();
  }

  /**
   * {@inheritdoc}
   */
  public function setActionUid(int $uid): self {
    $this->set('uid_action', $uid);

    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getActionTime(): int {
    return (int) $this->get('time_action')->first()?->getValue()['value'];
  }

  /**
   * {@inheritdoc}
   */
  public function setActionTime(int $time): self {
    $this->set('time_action', $time);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getActionType(): int {
    return (int) $this->get('type_action')->first()?->getValue()['value'];
  }

  /**
   * {@inheritdoc}
   */
  public function setActionType(int $type): self {
    $this->set('type_action', $type);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getActionMedium(): int {
    return (int) $this->get('medium_action')->first()?->getValue()['value'];
  }

  /**
   * {@inheritdoc}
   */
  public function setActionMedium(int $medium): self {
    $this->set('medium_action', $medium);
    return $this;
  }

}
