<?php

declare(strict_types=1);

namespace Drupal\l10n_server;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Database\StatementInterface;

/**
 * Provides a service to manage l10n server projects.
 */
class L10nProjectManager {

  /**
   * Constructs a new L10nProjectManager instance.
   *
   * @param \Drupal\Core\Database\Connection $connection
   *   The database connection.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cacheBackend
   *   The cache backend.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   */
  public function __construct(
    private readonly Connection $connection,
    private readonly CacheBackendInterface $cacheBackend,
    private readonly TimeInterface $time,
  ) {
  }

  /**
   * Build a list of initials for active projects.
   *
   * @return array
   *   An array of project initials.
   */
  public function getProjectInitials(): array {
    $cid = 'l10n_community_project_initials';
    if ($cache = $this->cacheBackend->get($cid)) {
      return (array) $cache->data;
    }

    $query = $this->connection
      ->query(
        'SELECT DISTINCT(SUBSTR(`title`, 1, 1)) AS initial FROM {l10n_server_project} WHERE `status` = :status ORDER BY initial',
        [':status' => 1]
      );

    if (!$query instanceof StatementInterface) {
      return [];
    }

    $project_initials = $query->fetchAllKeyed(0, 0);

    // Create an array of elements, all non-letters are grouped in '#'.
    $initials[] = [
      'title' => '#',
    ];

    foreach ($project_initials as $initial) {
      if (preg_match('/[a-zA-Z]/', $initial)) {
        $letter = strtoupper($initial);
        $initials[strtolower($letter)] = [
          'title' => $letter,
          'values' => [$letter],
        ];
        continue;
      }

      $initials[0]['values'][] = $initial;
    }

    $this->cacheBackend->set($cid, $initials, $this->time->getCurrentTime() + 86400);

    return $initials;
  }

}
